#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <string.h>
#include <sys/socket.h>
#include "bstack.h"
#include "imgmanip.h"
#include "history.h"
#include "sock.h"
#include "http.h"
#include "url.h"

/* TODO: remove unused headers after testing dynamic objects */

void showBitmap(SDL_Surface * screen, SDL_Surface * bitmap)
{
  if (SDL_BlitSurface(bitmap, NULL, screen, NULL) < 0)
    fprintf(stderr, "Failed to blit bitmap\n");

  SDL_Flip(screen);
}

int main(int argc, char ** argv)
{
  BoundedStack * history = BSCreate(10);
  SDL_Surface * screen = NULL;
  SDL_Surface * bitmap = NULL;
  int width = 300;
  int height = 200;
  int bpp = 32;
  SDL_Event event;
  int quit = 0;
  int keydown = 0;

  //parse command-line arguments if any
  if (argc == 2)
  {
    char * target = argv[1]; /* filename or URL */
    Url * url = NULL;

    /* check if filename is actually a URL */
    if (httpIsHttp(target))
    { /* retrieve image from URL */
      int sockfd = 0; /* used to connect to target URL */
      int code = 0; /* status code of the HTTP response */
      char * domain = NULL;

      url = urlCreate(target);
      domain = urlGetHost(url); /* domain part if URL */
      urlShowFields(url); /* for debugging only */

      sockfd = sockOpenTCP();
      if (sockConnect(sockfd, domain, 80) < 0)
      {
        printf("Connect failed!\n");
        urlDestroy(url);
        return -1;
      }
      else
      {
        httpSendGetRequest(sockfd, target);
        target = urlGetFilename(url);
        httpVerboseOn();
        code = httpRecvResponse(sockfd, target);

        if (code != 200)
        {
          printf("Failed to get remote file\n");
          remove(target);
          urlDestroy(url);
          return -1;
        }
      }
      sockClose(sockfd);
    }

    /* retrieve image from file */

    bitmap = IMG_Load(target);
    if (bitmap == NULL)
    {
      fprintf(stderr, "Failed to load image\n");
      return -1;
    }
    else
    {
      width = bitmap->w;
      height = bitmap->h;
    }

    if (httpIsHttp(argv[1]))
      urlDestroy(url);

  }

  //initialise SDL
  if (SDL_Init(SDL_INIT_VIDEO) < 0)
  {
    fprintf(stderr, "Failed to init SDL library\n");
    return -1;
  }

  //show window
  screen = SDL_SetVideoMode(width, height, bpp, SDL_SWSURFACE | SDL_DOUBLEBUF);
  if (screen == NULL)
  {
    fprintf(stderr, "Failed to show window\n");
    return -1;
  }

  //show bitmap
  showBitmap(screen, bitmap);
  historySave(screen, history); /* save current state of image */

  //set window caption
  SDL_WM_SetCaption("Picaxo", NULL);

  //parse user input
  while (quit == 0)
  {
    SDL_WaitEvent(&event);

    switch(event.type)
    {
      case SDL_KEYDOWN:
        keydown = 1;
        break;
      case SDL_KEYUP:
        keydown = 0;
        break;
      case SDL_QUIT: //X at top-right of window
        quit = 1;
        break;
    } //switch(event.type)

    if (keydown == 1)
    {
      switch(event.key.keysym.sym)
      {
        case SDLK_l:
          rotateLeft(screen, history);
          break;
        case SDLK_r:
          rotateRight(screen, history);
          break;
        case SDLK_v:
          flipVertical(screen, history);
          break;
        case SDLK_h:
          flipHorizontal(screen, history);
          break;
        case SDLK_n:
          negateImage(screen, history);
          break;
        case SDLK_g:
          grayscaleImage(screen, history);
          break;
        case SDLK_f:
          sandstormImage(screen, history);
          break;
        case SDLK_u:
          historyUndo(screen, history);
          break;
        case SDLK_y:
          historyRedo(screen, history);
          break;
        case SDLK_ESCAPE:
          quit = 1;
          break;
        default:
          break;
      }
    }

  } //while (quit == 0)

  //clean up
  BSDestroy(history);
  if (bitmap != NULL)
    SDL_FreeSurface(screen);
  SDL_FreeSurface(screen);
  SDL_Quit();
  return 0;
} 
